@@ -1,26 +1,29 @@
-/************************************************************************
- ************************************************************************
-    FAUST Architecture File
-    Copyright (C) 2003-2013 GRAME, Centre National de Creation Musicale
-    ---------------------------------------------------------------------
-    This Architecture section is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    as published by the Free Software Foundation; either version 3 of
-    the License, or (at your option) any later version.
+/************************** BEGIN CGlue.h *****************************
+FAUST Architecture File
+Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
+---------------------------------------------------------------------
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation; either version 2.1 of the License, or
+(at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; If not, see <http://www.gnu.org/licenses/>.
+You should have received a copy of the GNU Lesser General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- ************************************************************************
- ************************************************************************/
+EXCEPTION : As a special exception, you may create a larger work
+that contains this FAUST architecture section and distribute
+that work under terms of your choice, so long as this FAUST
+architecture section is not modified.
+*************************************************************************/
 
-#ifndef FAUST_UIGLUE_H
-#define FAUST_UIGLUE_H
+#ifndef CGLUE_H
+#define CGLUE_H
 
 #include "faust/gui/UI.h"
 #include "faust/gui/CInterface.h"
@@ -66,7 +69,7 @@ class UIFloat
     
         // -- soundfiles
     
-        virtual void addSoundFile(const char* label, const char* filename, Soundfile** sf_zone) = 0;
+        virtual void addSoundfile(const char* label, const char* filename, Soundfile** sf_zone) = 0;
 
         // -- metadata declarations
 
@@ -139,10 +142,10 @@ static void addVerticalBargraphGlueFloat(void* cpp_interface, const char* label,
     ui_interface->addVerticalBargraph(label, zone, min, max);
 }
     
-static void addSoundFileGlueFloat(void* cpp_interface, const char* label, const char* filename, Soundfile** sf_zone)
+static void addSoundfileGlueFloat(void* cpp_interface, const char* label, const char* url, Soundfile** sf_zone)
 {
     UIFloat* ui_interface = static_cast<UIFloat*>(cpp_interface);
-    ui_interface->addSoundFile(label, filename, sf_zone);
+    ui_interface->addSoundfile(label, url, sf_zone);
 }
 
 static void declareGlueFloat(void* cpp_interface, float* zone, const char* key, const char* value)
@@ -182,7 +185,7 @@ class UIDouble
     
         // -- soundfiles
     
-        virtual void addSoundFile(const char* label, const char* filename, Soundfile** sf_zone) = 0;
+        virtual void addSoundfile(const char* label, const char* filename, Soundfile** sf_zone) = 0;
 
         // -- metadata declarations
 
@@ -255,10 +258,10 @@ static void addVerticalBargraphGlueDouble(void* cpp_interface, const char* label
     ui_interface->addVerticalBargraph(label, zone, min, max);
 }
     
-static void addSoundFileGlueDouble(void* cpp_interface, const char* label, const char* filename, Soundfile** sf_zone)
+static void addSoundfileGlueDouble(void* cpp_interface, const char* label, const char* url, Soundfile** sf_zone)
 {
     UIDouble* ui_interface = static_cast<UIDouble*>(cpp_interface);
-    ui_interface->addSoundFile(label, filename, sf_zone);
+    ui_interface->addSoundfile(label, url, sf_zone);
 }
 
 static void declareGlueDouble(void* cpp_interface, double* zone, const char* key, const char* value)
@@ -267,173 +270,169 @@ static void declareGlueDouble(void* cpp_interface, double* zone, const char* key
     ui_interface->declare(zone, key, value);
 }
 
-inline void buildUIGlue(UIGlue* glue, UI* ui_interface, bool is_double)
+static void buildUIGlue(UIGlue* glue, UI* ui_interface, bool is_double)
 {
     glue->uiInterface = ui_interface;
     
     if (is_double) {
-        glue->openTabBox = (openTabBoxFun)openTabBoxGlueDouble;
-        glue->openHorizontalBox = (openHorizontalBoxFun)openHorizontalBoxGlueDouble;
-        glue->openVerticalBox = (openVerticalBoxFun)openVerticalBoxGlueDouble;
-        glue->closeBox = (closeBoxFun)closeBoxGlueDouble;
-        glue->addButton = (addButtonFun)addButtonGlueDouble;
-        glue->addCheckButton = (addCheckButtonFun)addCheckButtonGlueDouble;
-        glue->addVerticalSlider = (addVerticalSliderFun)addVerticalSliderGlueDouble;
-        glue->addHorizontalSlider = (addHorizontalSliderFun)addHorizontalSliderGlueDouble;
-        glue->addNumEntry = (addNumEntryFun)addNumEntryGlueDouble;
-        glue->addHorizontalBargraph = (addHorizontalBargraphFun)addHorizontalBargraphGlueDouble;
-        glue->addVerticalBargraph = (addVerticalBargraphFun)addVerticalBargraphGlueDouble;
-        glue->addSoundFile = (addSoundFileFun)addSoundFileGlueDouble;
-        glue->declare = (declareFun)declareGlueDouble;
+        glue->openTabBox = reinterpret_cast<openTabBoxFun>(openTabBoxGlueDouble);
+        glue->openHorizontalBox = reinterpret_cast<openHorizontalBoxFun>(openHorizontalBoxGlueDouble);
+        glue->openVerticalBox = reinterpret_cast<openVerticalBoxFun>(openVerticalBoxGlueDouble);
+        glue->closeBox = reinterpret_cast<closeBoxFun>(closeBoxGlueDouble);
+        glue->addButton = reinterpret_cast<addButtonFun>(addButtonGlueDouble);
+        glue->addCheckButton = reinterpret_cast<addCheckButtonFun>(addCheckButtonGlueDouble);
+        glue->addVerticalSlider = reinterpret_cast<addVerticalSliderFun>(addVerticalSliderGlueDouble);
+        glue->addHorizontalSlider = reinterpret_cast<addHorizontalSliderFun>(addHorizontalSliderGlueDouble);
+        glue->addNumEntry = reinterpret_cast<addNumEntryFun>(addNumEntryGlueDouble);
+        glue->addHorizontalBargraph = reinterpret_cast<addHorizontalBargraphFun>(addHorizontalBargraphGlueDouble);
+        glue->addVerticalBargraph = reinterpret_cast<addVerticalBargraphFun>(addVerticalBargraphGlueDouble);
+        glue->addSoundfile = reinterpret_cast<addSoundfileFun>(addSoundfileGlueDouble);
+        glue->declare = reinterpret_cast<declareFun>(declareGlueDouble);
     } else {
-        glue->openTabBox = (openTabBoxFun)openTabBoxGlueFloat;
-        glue->openHorizontalBox = (openHorizontalBoxFun)openHorizontalBoxGlueFloat;
-        glue->openVerticalBox = (openVerticalBoxFun)openVerticalBoxGlueFloat;
-        glue->closeBox = (closeBoxFun)closeBoxGlueFloat;
-        glue->addButton = (addButtonFun)addButtonGlueFloat;
-        glue->addCheckButton = (addCheckButtonFun)addCheckButtonGlueFloat;
-        glue->addVerticalSlider = (addVerticalSliderFun)addVerticalSliderGlueFloat;
-        glue->addHorizontalSlider = (addHorizontalSliderFun)addHorizontalSliderGlueFloat;
-        glue->addNumEntry = (addNumEntryFun)addNumEntryGlueFloat;
-        glue->addHorizontalBargraph = (addHorizontalBargraphFun)addHorizontalBargraphGlueFloat;
-        glue->addVerticalBargraph = (addVerticalBargraphFun)addVerticalBargraphGlueFloat;
-        glue->addSoundFile = (addSoundFileFun)addSoundFileGlueFloat;
-        glue->declare = (declareFun)declareGlueFloat;
+        glue->openTabBox = reinterpret_cast<openTabBoxFun>(openTabBoxGlueFloat);
+        glue->openHorizontalBox = reinterpret_cast<openHorizontalBoxFun>(openHorizontalBoxGlueFloat);
+        glue->openVerticalBox = reinterpret_cast<openVerticalBoxFun>(openVerticalBoxGlueFloat);
+        glue->closeBox = reinterpret_cast<closeBoxFun>(closeBoxGlueFloat);
+        glue->addButton = reinterpret_cast<addButtonFun>(addButtonGlueFloat);
+        glue->addCheckButton = reinterpret_cast<addCheckButtonFun>(addCheckButtonGlueFloat);
+        glue->addVerticalSlider = reinterpret_cast<addVerticalSliderFun>(addVerticalSliderGlueFloat);
+        glue->addHorizontalSlider = reinterpret_cast<addHorizontalSliderFun>(addHorizontalSliderGlueFloat);
+        glue->addNumEntry = reinterpret_cast<addNumEntryFun>(addNumEntryGlueFloat);
+        glue->addHorizontalBargraph = reinterpret_cast<addHorizontalBargraphFun>(addHorizontalBargraphGlueFloat);
+        glue->addVerticalBargraph = reinterpret_cast<addVerticalBargraphFun>(addVerticalBargraphGlueFloat);
+        glue->addSoundfile = reinterpret_cast<addSoundfileFun>(addSoundfileGlueFloat);
+        glue->declare = reinterpret_cast<declareFun>(declareGlueFloat);
     }
 }
     
-class UITemplate
+struct UITemplate
 {
     
-    private:
-        
-        void* fCPPInterface;
-        
-    public:
-        
-        UITemplate(void* cpp_interface):fCPPInterface(cpp_interface)
-        {}
-        
-        virtual ~UITemplate() {}
-        
-        // -- widget's layouts
-        
-        virtual void openTabBox(const char* label)
-        {
-            openTabBoxGlueFloat(fCPPInterface, label);
-        }
-        virtual void openHorizontalBox(const char* label)
-        {
-            openHorizontalBoxGlueFloat(fCPPInterface, label);
-        }
-        virtual void openVerticalBox(const char* label)
-        {
-            openVerticalBoxGlueFloat(fCPPInterface, label);
-        }
-        virtual void closeBox()
-        {
-            closeBoxGlueFloat(fCPPInterface);
-        }
-        
-        // float version
-        
-        // -- active widgets
-        
-        virtual void addButton(const char* label, float* zone)
-        {
-            addButtonGlueFloat(fCPPInterface, label, zone);
-        }
-        virtual void addCheckButton(const char* label, float* zone)
-        {
-            addCheckButtonGlueFloat(fCPPInterface, label, zone);
-        }
-        
-        virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step)
-        {
-            addVerticalSliderGlueFloat(fCPPInterface, label, zone, init, min, max, step);
-        }
-        
-        virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step)
-        {
-            addHorizontalSliderGlueFloat(fCPPInterface, label, zone, init, min, max, step);
-        }
-        
-        virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step)
-        {
-            addNumEntryGlueFloat(fCPPInterface, label, zone, init, min, max, step);
-        }
-        
-        // -- passive widgets
-        
-        virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max)
-        {
-            addHorizontalBargraphGlueFloat(fCPPInterface, label, zone, min, max);
-        }
-        
-        virtual void addVerticalBargraph(const char* label, float* zone, float min, float max)
-        {
-            addVerticalBargraphGlueFloat(fCPPInterface, label, zone, min, max);
-        }
+    void* fCPPInterface;
+
+    UITemplate(void* cpp_interface):fCPPInterface(cpp_interface)
+    {}
     
-        // -- metadata declarations
-        
-        virtual void declare(float* zone, const char* key, const char* val)
-        {
-            declareGlueFloat(fCPPInterface, zone, key, val);
-        }
-        
-        // double version
-        
-        virtual void addButton(const char* label, double* zone)
-        {
-            addButtonGlueDouble(fCPPInterface, label, zone);
-        }
-        virtual void addCheckButton(const char* label, double* zone)
-        {
-            addCheckButtonGlueDouble(fCPPInterface, label, zone);
-        }
-        
-        virtual void addVerticalSlider(const char* label, double* zone, double init, double min, double max, double step)
-        {
-            addVerticalSliderGlueDouble(fCPPInterface, label, zone, init, min, max, step);
-        }
-        
-        virtual void addHorizontalSlider(const char* label, double* zone, double init, double min, double max, double step)
-        {
-            addHorizontalSliderGlueDouble(fCPPInterface, label, zone, init, min, max, step);
-        }
-        
-        virtual void addNumEntry(const char* label, double* zone, double init, double min, double max, double step)
-        {
-            addNumEntryGlueDouble(fCPPInterface, label, zone, init, min, max, step);
-        }
-        
-        // -- passive widgets
-        
-        virtual void addHorizontalBargraph(const char* label, double* zone, double min, double max)
-        {
-            addHorizontalBargraphGlueDouble(fCPPInterface, label, zone, min, max);
-        }
-        
-        virtual void addVerticalBargraph(const char* label, double* zone, double min, double max)
-        {
-            addVerticalBargraphGlueDouble(fCPPInterface, label, zone, min, max);
-        }
+    virtual ~UITemplate() {}
     
-        // -- metadata declarations
-        
-        virtual void declare(double* zone, const char* key, const char* val)
-        {
-            declareGlueDouble(fCPPInterface, zone, key, val);
-        }
+    // -- widget's layouts
     
-        // -- soundfiles
+    virtual void openTabBox(const char* label)
+    {
+        openTabBoxGlueFloat(fCPPInterface, label);
+    }
+    virtual void openHorizontalBox(const char* label)
+    {
+        openHorizontalBoxGlueFloat(fCPPInterface, label);
+    }
+    virtual void openVerticalBox(const char* label)
+    {
+        openVerticalBoxGlueFloat(fCPPInterface, label);
+    }
+    virtual void closeBox()
+    {
+        closeBoxGlueFloat(fCPPInterface);
+    }
     
-        virtual void addSoundFile(const char* label, const char* filename, Soundfile** sf_zone)
-        {
-            addSoundFileGlueFloat(fCPPInterface, label, filename, sf_zone);
-        }
+    // float version
     
+    // -- active widgets
+    
+    virtual void addButton(const char* label, float* zone)
+    {
+        addButtonGlueFloat(fCPPInterface, label, zone);
+    }
+    virtual void addCheckButton(const char* label, float* zone)
+    {
+        addCheckButtonGlueFloat(fCPPInterface, label, zone);
+    }
+    
+    virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step)
+    {
+        addVerticalSliderGlueFloat(fCPPInterface, label, zone, init, min, max, step);
+    }
+    
+    virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step)
+    {
+        addHorizontalSliderGlueFloat(fCPPInterface, label, zone, init, min, max, step);
+    }
+    
+    virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step)
+    {
+        addNumEntryGlueFloat(fCPPInterface, label, zone, init, min, max, step);
+    }
+    
+    // -- passive widgets
+    
+    virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max)
+    {
+        addHorizontalBargraphGlueFloat(fCPPInterface, label, zone, min, max);
+    }
+    
+    virtual void addVerticalBargraph(const char* label, float* zone, float min, float max)
+    {
+        addVerticalBargraphGlueFloat(fCPPInterface, label, zone, min, max);
+    }
+
+    // -- metadata declarations
+    
+    virtual void declare(float* zone, const char* key, const char* val)
+    {
+        declareGlueFloat(fCPPInterface, zone, key, val);
+    }
+    
+    // double version
+    
+    virtual void addButton(const char* label, double* zone)
+    {
+        addButtonGlueDouble(fCPPInterface, label, zone);
+    }
+    virtual void addCheckButton(const char* label, double* zone)
+    {
+        addCheckButtonGlueDouble(fCPPInterface, label, zone);
+    }
+    
+    virtual void addVerticalSlider(const char* label, double* zone, double init, double min, double max, double step)
+    {
+        addVerticalSliderGlueDouble(fCPPInterface, label, zone, init, min, max, step);
+    }
+    
+    virtual void addHorizontalSlider(const char* label, double* zone, double init, double min, double max, double step)
+    {
+        addHorizontalSliderGlueDouble(fCPPInterface, label, zone, init, min, max, step);
+    }
+    
+    virtual void addNumEntry(const char* label, double* zone, double init, double min, double max, double step)
+    {
+        addNumEntryGlueDouble(fCPPInterface, label, zone, init, min, max, step);
+    }
+
+    // -- soundfiles
+    
+    virtual void addSoundfile(const char* label, const char* url, Soundfile** sf_zone)
+    {
+        addSoundfileGlueFloat(fCPPInterface, label, url, sf_zone);
+    }
+
+    // -- passive widgets
+    
+    virtual void addHorizontalBargraph(const char* label, double* zone, double min, double max)
+    {
+        addHorizontalBargraphGlueDouble(fCPPInterface, label, zone, min, max);
+    }
+    
+    virtual void addVerticalBargraph(const char* label, double* zone, double min, double max)
+    {
+        addVerticalBargraphGlueDouble(fCPPInterface, label, zone, min, max);
+    }
+
+    // -- metadata declarations
+    
+    virtual void declare(double* zone, const char* key, const char* val)
+    {
+        declareGlueDouble(fCPPInterface, zone, key, val);
+    }
+
 };
 
 /*******************************************************************************
@@ -446,7 +445,7 @@ static void declareMetaGlue(void* cpp_interface, const char* key, const char* va
     meta_interface->declare(key, value);
 }
 
-inline void buildMetaGlue(MetaGlue* glue, Meta* meta)
+static void buildMetaGlue(MetaGlue* glue, Meta* meta)
 {
     glue->metaInterface = meta;
     glue->declare = declareMetaGlue;
@@ -456,23 +455,23 @@ inline void buildMetaGlue(MetaGlue* glue, Meta* meta)
  * Memory manager glue code
  ******************************************************************************/
 
-static void* allocateManagerGlue(void* cpp_interface, size_t size)
+static void* allocateMemoryManagerGlue(void* cpp_interface, size_t size)
 {
     dsp_memory_manager* manager_interface = static_cast<dsp_memory_manager*>(cpp_interface);
     return manager_interface->allocate(size);
 }
     
-static void destroyManagerGlue(void* cpp_interface, void* ptr)
+static void destroyMemoryManagerGlue(void* cpp_interface, void* ptr)
 {
     dsp_memory_manager* manager_interface = static_cast<dsp_memory_manager*>(cpp_interface);
     manager_interface->destroy(ptr);
 }
 
-inline void buildManagerGlue(ManagerGlue* glue, dsp_memory_manager* manager)
+static void buildManagerGlue(MemoryManagerGlue* glue, dsp_memory_manager* manager)
 {
     glue->managerInterface = manager;
-    glue->allocate = allocateManagerGlue;
-    glue->destroy = destroyManagerGlue;
+    glue->allocate = allocateMemoryManagerGlue;
+    glue->destroy = destroyMemoryManagerGlue;
 }
 
 #ifdef __cplusplus
@@ -480,3 +479,4 @@ inline void buildManagerGlue(ManagerGlue* glue, dsp_memory_manager* manager)
 #endif
 
 #endif
+/**************************  END  CGlue.h **************************/
