@@ -1,26 +1,26 @@
-/************************** BEGIN midi.h **************************/
-/************************************************************************
- FAUST Architecture File
- Copyright (C) 2003-2017 GRAME, Centre National de Creation Musicale
- ---------------------------------------------------------------------
- This Architecture section is free software; you can redistribute it
- and/or modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 3 of
- the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; If not, see <http://www.gnu.org/licenses/>.
- 
- EXCEPTION : As a special exception, you may create a larger work
- that contains this FAUST architecture section and distribute
- that work under terms of your choice, so long as this FAUST
- architecture section is not modified.
- ************************************************************************/
+/************************** BEGIN midi.h *******************************
+FAUST Architecture File
+Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
+---------------------------------------------------------------------
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation; either version 2.1 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+EXCEPTION : As a special exception, you may create a larger work
+that contains this FAUST architecture section and distribute
+that work under terms of your choice, so long as this FAUST
+architecture section is not modified.
+************************************************************************/
 
 #ifndef __midi__
 #define __midi__
@@ -32,9 +32,9 @@
 
 class MapUI;
 
-/*************************************
- A time-stamped short MIDI message
-**************************************/
+/**
+ * A timestamped short MIDI message used with SOUL.
+ */
 
 // Force contiguous memory layout
 #pragma pack (push, 1)
@@ -45,15 +45,35 @@ struct MIDIMessage
 };
 #pragma pack (pop)
 
-/*******************************************************************************
+/**
+ * For timestamped MIDI messages.
+ */
+struct DatedMessage {
+    
+    double fDate;
+    unsigned char fBuffer[3];
+    size_t fSize;
+    
+    DatedMessage(double date, unsigned char* buffer, size_t size)
+    :fDate(date), fSize(size)
+    {
+        assert(size <= 3);
+        memcpy(fBuffer, buffer, size);
+    }
+    
+    DatedMessage():fDate(0.0), fSize(0)
+    {}
+    
+};
+
+/**
  * MIDI processor definition.
  *
  * MIDI input or output handling classes will implement this interface,
- * so the same method names (keyOn, ctrlChange...) will be used either
+ * so the same method names (keyOn, keyOff, ctrlChange...) will be used either
  * when decoding MIDI input or encoding MIDI output events.
  * MIDI channel is numbered in [0..15] in this layer.
- *******************************************************************************/
-
+ */
 class midi {
 
     public:
@@ -67,7 +87,7 @@ class midi {
             return keyOn(channel, pitch, velocity);
         }
         
-        virtual void keyOff(double, int channel, int pitch, int velocity = 127)
+        virtual void keyOff(double, int channel, int pitch, int velocity = 0)
         {
             keyOff(channel, pitch, velocity);
         }
@@ -157,10 +177,10 @@ class midi {
 
 };
 
-/*
- A class to decode NRPN and RPN messages, adapted from JUCE forum message: https://forum.juce.com/t/14bit-midi-controller-support/11517
-*/
-
+/**
+ * A class to decode NRPN and RPN messages, adapted from JUCE forum message:
+ * https://forum.juce.com/t/14bit-midi-controller-support/11517
+ */
 class MidiNRPN {
     
     private:
@@ -247,6 +267,14 @@ class MidiNRPN {
     
 };
 
+/**
+ * A pure interface for MIDI handlers that can send/receive MIDI messages to/from 'midi' objects.
+ */
+struct midi_interface {
+    virtual void addMidiIn(midi* midi_dsp)      = 0;
+    virtual void removeMidiIn(midi* midi_dsp)   = 0;
+    virtual ~midi_interface() {}
+};
 
 /****************************************************
  * Base class for MIDI input handling.
@@ -257,8 +285,7 @@ class MidiNRPN {
  * - decoding two data byte messages: handleData2
  * - getting ready messages in polling mode
  ****************************************************/
-
-class midi_handler : public midi {
+class midi_handler : public midi, public midi_interface {
 
     protected:
 
@@ -270,7 +297,7 @@ class midi_handler : public midi {
   
     public:
 
-        midi_handler(const std::string& name = "MIDIHandler"):fName(name) {}
+        midi_handler(const std::string& name = "MIDIHandler"):midi_interface(), fName(name) {}
         virtual ~midi_handler() {}
 
         void addMidiIn(midi* midi_dsp) { if (midi_dsp) fMidiInputs.push_back(midi_dsp); }
@@ -439,27 +466,5 @@ class midi_handler : public midi {
   
 };
 
-//-------------------------------
-// For timestamped MIDI messages
-//-------------------------------
-
-struct DatedMessage {
-    
-    double fDate;
-    unsigned char fBuffer[3];
-    size_t fSize;
-    
-    DatedMessage(double date, unsigned char* buffer, size_t size)
-    :fDate(date), fSize(size)
-    {
-        assert(size <= 3);
-        memcpy(fBuffer, buffer, size);
-    }
-    
-    DatedMessage():fDate(0.0), fSize(0)
-    {}
-    
-};
-
 #endif // __midi__
 /**************************  END  midi.h **************************/
