@@ -22,49 +22,24 @@
 
 #import <QuartzCore/QuartzCore.h>
 #import "FIMainViewController.h"
-#import "ios-faust.h"
 #import "FIFlipsideViewController.h"
 #import "FIAppDelegate.h"
 
-#include "faust/dsp/timed-dsp.h"
-#include "faust/gui/JSONUI.h"
-#include "faust/audio/coreaudio-ios-dsp.h"
-
-#if OSCCTRL
-#include "faust/gui/OSCUI.h"
-#endif
-
-#if MIDICTRL
-#include "faust/gui/MidiUI.h"
-#endif
+// faust -i (options) generated file
+#import "ios-faust.h"
 
 #define kMenuBarsHeight             66
 #define kMotionUpdateRate           30
-
 #define kRefreshTimerInterval       0.04
-
 #define ONE_G 9.81
 
-// Test Jack
-#define kJackViewHeight 130
-#define kJackViewAnimationDuration 0.2
-
 #define SYSTEM_VERSION_EQUAL_TO(v)                  ([[[UIDevice currentDevice] systemVersion] compare:v options:NSNumericSearch] == NSOrderedSame)
 #define SYSTEM_VERSION_GREATER_THAN(v)              ([[[UIDevice currentDevice] systemVersion] compare:v options:NSNumericSearch] == NSOrderedDescending)
 #define SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(v)  ([[[UIDevice currentDevice] systemVersion] compare:v options:NSNumericSearch] != NSOrderedAscending)
 #define SYSTEM_VERSION_LESS_THAN(v)                 ([[[UIDevice currentDevice] systemVersion] compare:v options:NSNumericSearch] == NSOrderedAscending)
 #define SYSTEM_VERSION_LESS_THAN_OR_EQUAL_TO(v)     ([[[UIDevice currentDevice] systemVersion] compare:v options:NSNumericSearch] != NSOrderedDescending)
 
-#include "faust/dsp/poly-dsp.h"
-
-#if POLY2
-#include "faust/dsp/dsp-combiner.h"
-#include "effect.cpp"
-#endif
-
 #if MIDICTRL
-#include "faust/midi/rt-midi.h"
-#include "faust/midi/RtMidi.cpp"
 rt_midi* midi_handler;
 MidiUI* midiinterface = NULL;
 #endif
@@ -82,13 +57,18 @@ ztimedmap GUI::gTimedZoneMap;
 audio* audio_device = NULL;
 CocoaUI* uiinterface = NULL;
 FUI* finterface = NULL;
+SaveLabelUI* saveinterface = NULL;
 
 #if SOUNDFILE
 SoundUI* soundinterface = NULL;
 #endif
 
 #if OSCCTRL
-GUI* oscinterface = NULL;
+OSCUI* oscinterface = NULL;
+static void osc_compute_callback(void* arg)
+{
+    oscinterface->endBundle();
+}
 #endif
 
 MY_Meta metadata;
@@ -98,6 +78,7 @@ int sample_rate = 0;
 int buffer_size = 0;
 BOOL openWidgetPanel = YES;
 int uiCocoaItem::gItemCount = 0;
+float uiBox::gDummy = 0;
 
 - (void)didReceiveMemoryWarning
 {
@@ -111,18 +92,6 @@ int uiCocoaItem::gItemCount = 0;
     [super loadView];
 }
 
-#ifdef JACK_IOS
-
-static void jack_shutdown_callback(const char* message, void* arg)
-{
-    FIMainViewController* self = (FIMainViewController*)arg;
-    dispatch_async(dispatch_get_main_queue(), ^{
-        [self closeJack :message];
-    });
-}
-
-#endif
-
 - (void)viewDidLoad
 {
     // General UI initializations
@@ -136,41 +105,38 @@ static void jack_shutdown_callback(const char* message, void* arg)
     
     bool midi_sync = false;
     int nvoices = 0;
-    mydsp_poly* dsp_poly = NULL;
     
+#if MIDICTRL
     mydsp* tmp_dsp = new mydsp();
     MidiMeta::analyse(tmp_dsp, midi_sync, nvoices);
     delete tmp_dsp;
+ #endif
     
 #if POLY2
     bool group = true;
     std::cout << "Started with " << nvoices << " voices\n";
-    dsp_poly = new mydsp_poly(new mydsp(), nvoices, true, group);
+    DSP = new mydsp_poly(new mydsp(), nvoices, true, group);
     
     #if MIDICTRL
     if (midi_sync) {
-        DSP = new timed_dsp(new dsp_sequencer(dsp_poly, new effect()));
+        DSP = new timed_dsp(new dsp_sequencer(DSP, new effect()));
     } else {
-        DSP = new dsp_sequencer(dsp_poly, new effect());
+        DSP = new dsp_sequencer(DSP, new effect());
     }
     #else
-        DSP = new dsp_sequencer(dsp_poly, new effect());
+        DSP = new dsp_sequencer(DSP, new effect());
     #endif
 #else
     bool group = true;
     
     if (nvoices > 0) {
         std::cout << "Started with " << nvoices << " voices\n";
-        dsp_poly = new mydsp_poly(new mydsp(), nvoices, true, group);
+        DSP = new mydsp_poly(new mydsp(), nvoices, true, group);
         
     #if MIDICTRL
         if (midi_sync) {
-            DSP = new timed_dsp(dsp_poly);
-        } else {
-            DSP = dsp_poly;
+            DSP = new timed_dsp(DSP);
         }
-    #else
-        DSP = dsp_poly;
     #endif
     } else {
     #if MIDICTRL
@@ -202,9 +168,10 @@ static void jack_shutdown_callback(const char* message, void* arg)
     
     uiinterface = new CocoaUI([UIApplication sharedApplication].keyWindow, self, &metadata, DSP);
     finterface = new FUI();
+    saveinterface = new SaveLabelUI();
+    
 #if MIDICTRL
     midi_handler = new rt_midi(_name);
-    midi_handler->addMidiIn(dsp_poly);
     midiinterface = new MidiUI(midi_handler);
 #endif
       
@@ -227,15 +194,13 @@ static void jack_shutdown_callback(const char* message, void* arg)
     oscOutputPortText = [[NSUserDefaults standardUserDefaults] stringForKey:@"oscOutputPortText"];
     oscOutputPortText = (oscOutputPortText) ? oscOutputPortText : @"5511";
     
-    [self openAudio];
-    
-    // Build Faust interface
-    DSP->init(int(sample_rate));
     DSP->buildUserInterface(uiinterface);
     DSP->buildUserInterface(finterface);
+    DSP->buildUserInterface(saveinterface);
     
 #if SOUNDFILE
-    SoundUI* soundinterface = new SoundUI();
+    // Use bundle path
+    soundinterface = new SoundUI(SoundUI::getBinaryPath());
     DSP->buildUserInterface(soundinterface);
 #endif
     
@@ -243,6 +208,11 @@ static void jack_shutdown_callback(const char* message, void* arg)
     DSP->buildUserInterface(midiinterface);
 #endif
     
+    [self openAudio];
+    
+    // Build Faust interface
+    DSP->init(int(sample_rate));
+    
     [self displayTitle];
 
 #if MIDICTRL
@@ -269,7 +239,7 @@ static void jack_shutdown_callback(const char* message, void* arg)
     // Abstract layout is the layout computed without regarding screen dimensions. To be displayed, we adapt it to the device and orientation
     uiinterface->saveAbstractLayout();
     
-    // Used to refresh bargraphes
+    // Used to refresh bargraphs
     _refreshTimer = [NSTimer scheduledTimerWithTimeInterval:kRefreshTimerInterval target:self selector:@selector(refreshObjects:) userInfo:nil repeats:YES];
     
     // Views initilizations
@@ -289,119 +259,10 @@ static void jack_shutdown_callback(const char* message, void* arg)
     _motionManager = nil;
     _selectedWidget = nil;
     [self loadWidgetsPreferences];
+    
     if (_assignatedWidgets.size() > 0 || uiinterface->isScreenUI()) [self startMotion];
-
-#ifdef JACK_IOS
-    // Test Jack
-    _swipeRecognizer = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(openJackView)];
-    _swipeRecognizer.direction = UISwipeGestureRecognizerDirectionUp;
-    _swipeRecognizer.numberOfTouchesRequired = 3;
-    [_dspScrollView addGestureRecognizer:_swipeRecognizer];
-    
-    _tapRecognizerToDismissJackView = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(closeJackView)];
-    _tapRecognizerToDismissJackView.numberOfTapsRequired = 1;
-    _tapRecognizerToDismissJackView.numberOfTouchesRequired = 1;
-    [_dspScrollView addGestureRecognizer:_tapRecognizerToDismissJackView];
-    
-    _jackButton = [UIButton buttonWithType:UIButtonTypeCustom];
-    [_jackButton setImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Icon-Jack" ofType:@"png"]]
-                 forState:UIControlStateNormal];
-    [_jackButton addTarget:self action:@selector(openJackView) forControlEvents:UIControlEventTouchUpInside];
-
-    _jackButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin & UIViewAutoresizingFlexibleTopMargin;
-    [self.view addSubview:_jackButton];
-    
-    [self.view bringSubviewToFront:_widgetPreferencesView];
-    
-    _jackView = nil;
-    _orientationIsChanging = NO;
-#endif
 }
 
-#ifdef JACK_IOS
-
-- (BOOL)checkJack
-{
-    jackaudio audio;
-    
-    if (audio.init("dummy", &DSP)) {
-        audio.stop();
-        return TRUE;
-    } else {
-        return FALSE;
-    }
-}
-
-- (BOOL)openJack
-{
-    if (!audio_device) {
-        
-        NSString* iconFile;
-        if (DSP->getNumInputs() > 0 && DSP->getNumOutputs() > 0) {
-            iconFile = [[NSBundle mainBundle] pathForResource:@"Icon-Fx136" ofType:@"png"];
-        } else if (DSP->getNumOutputs() > 0) {
-            iconFile = [[NSBundle mainBundle] pathForResource:@"Icon-Output136" ofType:@"png"];
-        } else {
-            iconFile = [[NSBundle mainBundle] pathForResource:@"Icon-Analyzer136" ofType:@"png"];
-        }
-        NSFileHandle* fileHandle = [NSFileHandle fileHandleForReadingAtPath:iconFile];
-        NSData* data = [fileHandle readDataToEndOfFile];
-        const void* icon_data = [data bytes];
-        const size_t size = [data length];
-        NSLog(@"publishAppIcon rawDataSize = %ld", size);
-        [fileHandle closeFile];
-        
-        audio_device = new jackaudio(icon_data, size, true);
-        if (!audio_device->init((_name) ? _name : "Faust", DSP)) {
-            printf("Cannot connect to JACK server\n");
-            goto error;
-        }
-        
-        if (audio_device->start() < 0) {
-            printf("Cannot start JACK client\n");
-            goto error;
-        }
-    }
-    
-    audio_device->shutdown(jack_shutdown_callback, self);
-    return TRUE;
-    
-error:
-    
-    UIAlertView* alertView = [[UIAlertView alloc] initWithTitle:@"Audio warning"
-                                                        message:@"JACK server is not running !" delegate:self
-                                              cancelButtonTitle:@"OK" otherButtonTitles:nil];
-    [alertView show];
-    [alertView release];
-    
-    [self closeAudio];
-    return FALSE;
-}
-
-// Save widgets values
-- (void)closeJack:(const char*)reason 
-{
-    NSString* errorString = [[NSString alloc] initWithCString:reason encoding:NSASCIIStringEncoding];
-    
-    if ([errorString compare:@"Client closed from JACK server!"] != NSOrderedSame)
-    {
-        UIAlertView* alertView = [[UIAlertView alloc] initWithTitle:@"Audio error"
-                                                            message:errorString delegate:self
-                                                            cancelButtonTitle:@"OK" otherButtonTitles:nil];
-        [alertView show];
-        [alertView release];
-    }
-
-    [self closeAudio];
-}
-
-- (BOOL)isJackAudio
-{
-    return (dynamic_cast<jackaudio*>(audio_device) != NULL);
-}
-
-#endif
-
 - (BOOL)openCoreAudio:(int)bufferSize :(int)sampleRate
 {
     if (!audio_device) {
@@ -432,29 +293,11 @@ error:
     return FALSE;
 }
 
-#ifdef JACK_IOS
-
-- (void)openAudio
-{
-    // If CA audio running and JACK server running, will switch to JACK
-    if (audio_device && dynamic_cast<iosaudio*>(audio_device) && [self checkJack]) {
-        [self closeAudio];
-    }
-    
-    if (![self openJack]) {
-        [self openCoreAudio:bufferSize :sample_rate];
-    }
-}
-
-#else
-
 - (void)openAudio
 {
     [self openCoreAudio:buffer_size :sample_rate];
 }
 
-#endif
-
 - (void)closeAudio
 {
     if (audio_device) {
@@ -523,6 +366,7 @@ error:
     
     delete uiinterface;
     delete finterface;
+    delete saveinterface;
     
 #if SOUNDFILE
     delete soundinterface;
@@ -542,11 +386,6 @@ error:
     [_refreshTimer invalidate];
     [self stopMotion];
 
-#ifdef JACK_IOS
-    // Test Jack
-    [_swipeRecognizer release];
-#endif
-    
     [_curveSegmentedControl release];
     [_minText release];
     [_maxText release];
@@ -714,16 +553,8 @@ T findCorrespondingUiItem(FIResponder* sender)
         return;
     }
     
-#ifdef JACK_IOS
-    _orientationIsChanging = YES;
-#endif
-    
     [self updateGui];
     
-#ifdef JACK_IOS
-    _orientationIsChanging = NO;
-#endif
-    
     // Compute layout
     if (deviceOrientation == UIDeviceOrientationPortrait
         || deviceOrientation == UIDeviceOrientationPortraitUpsideDown)
@@ -817,19 +648,13 @@ T findCorrespondingUiItem(FIResponder* sender)
             // TO min/mid/max
         }
     }
-
-#ifdef JACK_IOS    
-    // Test Jack
-    if ([self isJackAudio])
-    {
-        [self performSelector:@selector(autoResizeJackViews) withObject:nil afterDelay:0.1];
-    }
-#endif
 }
 
 // Locked box : box currently zoomed in
 - (void)zoomToLockedBox
 {
+    if (!_lockedBox) return;
+    
     if (_lockedBox == uiinterface->getMainBox())
     {
         [_dspScrollView setZoomScale:_dspScrollView.minimumZoomScale animated:YES];
@@ -945,11 +770,7 @@ T findCorrespondingUiItem(FIResponder* sender)
 }
 
 - (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
-{
-#ifdef JACK_IOS
-    [self closeJackView];
-#endif
-}
+{}
 
 // Function called just after scroll view scrolled
 - (void)scrollViewDidScroll:(UIScrollView *)scrollView
@@ -970,11 +791,6 @@ T findCorrespondingUiItem(FIResponder* sender)
 // User just double tapped somewhere in the DSP view
 - (void)doubleTap
 {
-#ifdef JACK_IOS
-    // Test Jack
-    [self closeJackView];
-#endif
-    
     uiBox* tapedBox = uiinterface->getBoxForPoint([_tapGesture locationInView:_dspView]);
 
     // Avoid a strange bug
@@ -1063,6 +879,9 @@ static inline const char* transmit_value(int num)
 - (void)setOSCParameters:(int)transmit output:(NSString*)outputIPText inputport:(NSString*)inputPortText outputport:(NSString*)outputPortText;
 {
 #if OSCCTRL
+    // Save current stare;
+    saveinterface->save();
+    
     delete oscinterface;
     const char* argv[9];
     argv[0] = (char*)_name;
@@ -1074,8 +893,22 @@ static inline const char* transmit_value(int num)
     argv[6] = [inputPortText cStringUsingEncoding:[NSString defaultCStringEncoding]];
     argv[7] = "-outport";
     argv[8] = [outputPortText cStringUsingEncoding:[NSString defaultCStringEncoding]];
+    
+    /*
+    // Deactivated for now (sometimes crashing)
+    argv[9] = "-bundle";
+    argv[10] = "1";
+    */
+    
+    // Start OSC interface
     oscinterface = new OSCUI(_name, 9, (char**)argv);
     DSP->buildUserInterface(oscinterface);
+    audio_device->addControlCallback(osc_compute_callback, self);
+    
+    // Load current controller state
+    DSP->buildUserInterface(saveinterface);
+    
+    // Start OSC
     oscinterface->run();
 #endif
 }
@@ -1259,10 +1092,6 @@ static inline const char* transmit_value(int num)
     [animation setTimingFunction:[CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut]];	
     
     [[_dspView.window layer] addAnimation:animation forKey:@"ShowWidgetPreferences"];
- 
-#ifdef JACK_IOS
-    [self closeJackView];
-#endif
 }
 
 // Display right values for parameters
@@ -1699,7 +1528,7 @@ static inline const char* transmit_value(int num)
             printf("startGyroUpdates\n");
         }
         
-        if (_hasAcc || _hasGyr) {
+        if (_hasAcc || _hasGyr || uiinterface->isScreenUI()) {
             _motionTimer = [NSTimer scheduledTimerWithTimeInterval:1./kMotionUpdateRate
                                                             target:self
                                                           selector:@selector(updateMotion)
@@ -1781,102 +1610,4 @@ static inline const char* transmit_value(int num)
     return result;
 }
 
-#ifdef JACK_IOS
-// Test Jack
-- (void)openJackView
-{
-    if (_jackView) return;
-    
-    // Construct view
-    _jackView = [[JackView alloc] initWithFrame:CGRectMake(0,
-                                                           _dspScrollView.frame.size.height,
-                                                           _dspScrollView.frame.size.width,
-                                                           kJackViewHeight)];
-    _jackView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
-    
-    // Load client in view
-    [_jackView loadJackClient:dynamic_cast<jackaudio*>(audio_device)->get_client()];
-    
-    // Insert view in super view
-    [_dspScrollView.superview addSubview:_jackView];
-    
-    // Gesture recognizers
-    [_swipeRecognizer removeTarget:self action:@selector(openJackView)];
-    [_swipeRecognizer addTarget:self action:@selector(closeJackView)];
-    _swipeRecognizer.direction = UISwipeGestureRecognizerDirectionDown;
-    
-    // Animation
-    [UIView animateWithDuration:kJackViewAnimationDuration
-                    delay:0.0
-                    options:UIViewAnimationOptionCurveLinear
-                    animations:^
-     {
-         [_jackView setFrame:CGRectMake(0,
-                                        _dspScrollView.frame.size.height - kJackViewHeight + 44,
-                                        _dspScrollView.frame.size.width,
-                                        kJackViewHeight)];
-     }
-                     completion:^(BOOL finished)
-     {
-     }];
-}
-
-- (void)closeJackView
-{
-    if (!_jackView) return;
-    
-    // Gesture recognizers
-    [_swipeRecognizer removeTarget:self action:@selector(closeJackView)];
-    [_swipeRecognizer addTarget:self action:@selector(openJackView)];
-    _swipeRecognizer.direction = UISwipeGestureRecognizerDirectionUp;
-        
-    // Animation
-    [UIView animateWithDuration:kJackViewAnimationDuration
-                          delay:0.0
-                        options:UIViewAnimationOptionCurveLinear
-                     animations:^
-     {
-         [_jackView setFrame:CGRectMake(0,
-                                        _dspScrollView.frame.size.height + 44,
-                                        _dspScrollView.frame.size.width,
-                                        kJackViewHeight)];
-     }
-                     completion:^(BOOL finished)
-     {
-         [_jackView removeFromSuperview];
-         [_jackView release];
-         _jackView = nil;
-     }];
-}
-
-- (void)autoResizeJackViews
-{
-    if (_jackView)
-    {
-        [_jackView setFrame:CGRectMake(0,
-                                       _dspScrollView.frame.size.height - kJackViewHeight + 44,
-                                       _dspScrollView.frame.size.width,
-                                       kJackViewHeight)];
-    }
-    
-    if (_jackButton)
-    {
-        if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
-        {
-            [_jackButton setFrame:CGRectMake(_dspScrollView.frame.size.width - 70 - 50,
-                                             _dspScrollView.frame.size.height,
-                                             70,
-                                             32)];
-        }
-        else
-        {
-            [_jackButton setFrame:CGRectMake(_dspScrollView.frame.size.width - 70 - 10,
-                                             _dspScrollView.frame.size.height,
-                                             70,
-                                             32)];
-        }
-    }
-}
-
-#endif
 @end
