@@ -1,11 +1,12 @@
-// @notes MODEL, RELEVANTJSONFILE, PLUGNAME and ANDROIDPLUGINNAME are replaced during processing
+// @notes MODEL, RELEVANTJSONFILE, PLUGNAME and PLUGINNAME are replaced during processing
 using System;
 using System.Text;
 using System.Runtime.InteropServices;
 using System.Collections.Generic;
+using System.Globalization;
 using UnityEngine;
 
-/* @brief This file contains additionnal codes to communicate with the plugin and build the inspector interface
+/* @brief This file contains additional code to communicate with the plugin and build the inspector interface
 * @brief It shouldn't be changed
 */
 namespace FaustUtilities_MODEL {
@@ -75,7 +76,7 @@ namespace FaustUtilities_MODEL {
 		private void progressBar(float value, Group item) {
 			EditorGUILayout.Space();
 			EditorGUILayout.BeginHorizontal();
-			if (item.label == "0x00") { // If the name of the parameter haven't been set, the default name 0x00 is erased
+			if (item.label == "0x00") { // If the name of the parameter hasn't been set, the default name 0x00 is erased
 				item.label = "";
 			}
 			Rect rect = GUILayoutUtility.GetRect(18, 18, "TextField");
@@ -85,13 +86,13 @@ namespace FaustUtilities_MODEL {
 			EditorGUILayout.Space();
 		}
 
-		/* @brief Method to display an horizontal slider
+		/* @brief Method to display a horizontal slider
         * @param value Current value of the param in the dsp
         * @param item Which item is displayed
         * @return the new value of the param */
 		private float hSlider(float value, Group item) {
 			EditorGUILayout.BeginHorizontal();
-			if (item.label == "0x00") { // If the name of the parameter haven't been set, the default name 0x00 is erased
+			if (item.label == "0x00") { // If the name of the parameter hasn't been set, the default name 0x00 is erased now
 				item.label = "";
 			}
 			float newvalue = EditorGUILayout.Slider(new GUIContent(item.label, helpBox(item)), value, item.min, item.max);
@@ -106,7 +107,7 @@ namespace FaustUtilities_MODEL {
         * @return the new value of the entry */
 		private float numEntry(float value, Group item) {
 			EditorGUILayout.BeginHorizontal();
-			if (item.label == "0x00") { // If the name of the parameter haven't been set, the default name 0x00 is erased
+			if (item.label == "0x00") { // If the name of the parameter hasn't been set, the default name 0x00 is erased
 				item.label = "";
 			}
 			float newvalue = EditorGUILayout.FloatField(new GUIContent(item.label, helpBox(item)), value);
@@ -121,7 +122,7 @@ namespace FaustUtilities_MODEL {
         * @return the new value of the param*/
 		private float checkBox(float value, Group item) {
 			EditorGUILayout.BeginHorizontal();
-			if (item.label == "0x00") { // If the name of the parameter haven't been set, the default name 0x00 is erased
+			if (item.label == "0x00") { // If the name of the parameter hasn't been set, the default name 0x00 is erased
 				item.label = "";
 			}
 			// A conversion between bool and float needs to be done (Unity: bool; Faust: float)
@@ -139,7 +140,7 @@ namespace FaustUtilities_MODEL {
         * @return the new value if the button*/
 		private float button(Group item) {
 			EditorGUILayout.BeginHorizontal();
-			if (item.label == "0x00") { // If the name of the parameter haven't been set, the default name 0x00 is erased
+			if (item.label == "0x00") { // If the name of the parameter hasn't been set, the default name 0x00 is erased
 				item.label = "";
 			}
 			if (GUILayout.Button(new GUIContent(item.label, helpBox(item)))) {
@@ -175,7 +176,7 @@ namespace FaustUtilities_MODEL {
 			}
 		}
 
-		/* @brief Method to add the different elements on the inspector and deals with the varition of the parameter
+		/* @brief Method to add the different elements on the inspector and deal with parameter variation
         * @param item Which item will be displayed */
 		private void addComponent(Group item) {
 			int numparam = item.numparam;
@@ -245,13 +246,15 @@ namespace FaustUtilities_MODEL {
 
 		private IntPtr _context;
 
-        #if UNITY_STANDALONE_OSX || UNITY_STANDALONE_WIN || UNITY_EDITOR || UNITY_STANDALONE_LINUX || UNITY_WSA || UNITY_WSA_10_0 || UNITY_IOS
-        const string _dllName = "PLUGNAME";
-        #elif UNITY_ANDROID
-        const string _dllName = "ANDROIDPLUGINNAME";
-        #else
-        Debug.LogError("Architecture not supported by the plugin");
-        #endif
+		#if UNITY_EDITOR_OSX || UNITY_EDITOR_WIN || UNITY_STANDALONE_OSX || UNITY_STANDALONE_WIN || UNITY_WSA || UNITY_WSA_10_0
+		const string _dllName = "PLUGNAME";
+		#elif UNITY_IOS
+		const string _dllName = "__Internal";
+		#elif UNITY_EDITOR || UNITY_ANDROID || UNITY_STANDALONE_LINUX
+		const string _dllName = "PLUGINNAME";
+		#else
+		Debug.LogError("Architecture not supported by the plugin");
+		#endif
 
 		// Imports all c++ function to intialize and process the dsp. The methods need to be private
 		[DllImport(_dllName)]
@@ -334,7 +337,7 @@ namespace FaustUtilities_MODEL {
 	}
 
 	/* @brief Class to parse the JSON
-	* @brief These attributes represent the global datas except the global metadatas (useless in this context)
+	* @brief These attributes represent the global data except the global metadata (useless in this context)
 	* @brief Also contains all methods to parse the JSON
 	*/
 	public class FaustUI {
@@ -360,19 +363,34 @@ namespace FaustUtilities_MODEL {
 							success = parseDQString(ref fJSON, out value);
 							faustUI.name = value.ToString();
 							break;
+                        case "filename":            // Don't use the result
+                            success = parseDQString(ref fJSON, out value);
+                            break;
+                        case "version":             // Don't use the result
+                            success = parseDQString(ref fJSON, out value);
+                            break;
+                        case "compile_options":     // Don't use the result
+                            success = parseDQString(ref fJSON, out value);
+                            break;
+                        case "library_list":        // Don't use the result
+                            success = parseGlobalMetaData(ref fJSON);
+                            break;
+                        case "include_pathnames":   // Don't use the result
+                            success = parseGlobalMetaData(ref fJSON);
+                            break;
 						case "inputs":
-							success = parseDQString(ref fJSON, out value);
+							success = parseNum(ref fJSON,  out value);
 							faustUI.inputs = Convert.ToInt32(value.ToString());
 							break;
 						case "outputs":
-							success = parseDQString(ref fJSON, out value);
+							success = parseNum(ref fJSON,  out value);
 							faustUI.outputs = Convert.ToInt32(value.ToString());
 							break;
 						case "meta":
 							success = parseGlobalMetaData(ref fJSON);
 							break;
-						case "ui":
-                     		faustUI.ui = new List < Group > ();
+	                    case "ui":
+							faustUI.ui = new List < Group > ();
 							int numitems = 0;
 							success = parseUI(ref fJSON, ref faustUI.ui, ref numitems);
 							break;
@@ -425,6 +443,22 @@ namespace FaustUtilities_MODEL {
 			return valid;
 		}
 
+		private static bool parseNum(ref string s,  out StringBuilder name) {
+			bool valid = false;
+			name = new StringBuilder();
+			char[] c = s.ToCharArray();
+			if (c[0] == ' ') {
+				int i = 1;
+				while (c[i] != ' ' && c[i] != ',' && c[i] != '}' && c[i] != ']') {
+					name.Append(c[i]);
+					i++;
+				}
+				valid = true;
+				s = skipChar(ref s, name.ToString().Length + 1);
+			}
+			return valid;
+		}
+
 		private static bool parseDQString(ref string s, out StringBuilder word) {
 			return parseString(ref s, '"', out word);
 		}
@@ -478,6 +512,7 @@ namespace FaustUtilities_MODEL {
 		}
 
 		private static bool parseUI(ref string s, ref List < Group > uiItems, ref int numItems) {
+			CultureInfo culture = new CultureInfo("en-US");
 			if (parseChar(ref s, '[')) {
 
 				StringBuilder label;
@@ -516,26 +551,26 @@ namespace FaustUtilities_MODEL {
 									}
 									break;
 								case "init":
-									if (parseChar(ref s, ':') && parseDQString(ref s, out value)) {
-										float x = Convert.ToSingle(value.ToString());
+									if (parseChar(ref s, ':') && parseNum(ref s, out value)) {
+										float x = Convert.ToSingle(value.ToString(), culture);
 										uiItems[numItems].init = x;
 									}
 									break;
 								case "min":
-									if (parseChar(ref s, ':') && parseDQString(ref s, out value)) {
-										float x = Convert.ToSingle(value.ToString());
+									if (parseChar(ref s, ':') && parseNum(ref s, out value)) {
+										float x = Convert.ToSingle(value.ToString(), culture);
 										uiItems[numItems].min = x;
 									}
 									break;
 								case "max":
-									if (parseChar(ref s, ':') && parseDQString(ref s, out value)) {
-										float x = Convert.ToSingle(value.ToString());
+									if (parseChar(ref s, ':') && parseNum(ref s, out value)) {
+										float x = Convert.ToSingle(value.ToString(), culture);
 										uiItems[numItems].max = x;
 									}
 									break;
 								case "step":
-									if (parseChar(ref s, ':') && parseDQString(ref s, out value)) {
-										float x = Convert.ToSingle(value.ToString());
+									if (parseChar(ref s, ':') && parseNum(ref s, out value)) {
+										float x = Convert.ToSingle(value.ToString(), culture);
 										uiItems[numItems].step = x;
 									}
 									break;
@@ -552,8 +587,6 @@ namespace FaustUtilities_MODEL {
 							}
 						} while (parseChar(ref s , ','));
 						parseChar(ref s, '}');
-					} else {
-						return false;
 					}
 				} while (parseChar(ref s , ','));
 				return parseChar(ref s, ']');
@@ -614,12 +647,11 @@ namespace FaustUtilities_MODEL {
 		}
 	}
 
-	/* @brief Class to parse the metadatas of an item (not the global metadatas)
+	/* @brief Class to parse the metadata of an item (not the global metadata)
 	*/
 	public class Meta {
 		public string unit;
 		public string scale;
 		public string tooltip;
 	}
-
 }
